/****************************************************
 * 有機ＥＬキャラクタディスプレイモジュール　１６×２行
 * SO1602AWWB-UC-WB-U　（電源電圧3.3V）
 *　 表示テストプログラム
 * 　使用マイコン　PIC16F1827
 *   使用コンパイラ　XC8　Ver.1.45
 * 　I2C　割り当て
 * 　SDA  :  SDA1:PORTB(1) pin7
 * 　SCL  :  SCL1:PORTB(4) pin10
 * 
 *                                2021／1／25
 ****************************************************/

#include    <xc.h>
#include    <stdio.h>
#include    "i2c_lib4.h"

/************** コンフィグレーション設定   *****************/
// コンフィギュレーション１の設定
#pragma config FOSC = INTOSC    // 内部ｸﾛｯｸを使用する(INTOSC)
#pragma config WDTE = OFF       // ｳｵｯﾁﾄﾞｯｸﾞﾀｲﾏｰ無し(OFF)
#pragma config PWRTE = ON       // 電源ONから64ms後にﾌﾟﾛｸﾞﾗﾑを開始する(ON)
#pragma config MCLRE = ON       // 外部ﾘｾｯﾄ信号を使用
#pragma config CP = OFF         // ﾌﾟﾛｸﾞﾗﾑﾒﾓﾘｰを保護しない(OFF)
#pragma config CPD = OFF        // ﾃﾞｰﾀﾒﾓﾘｰを保護しない(OFF)
#pragma config BOREN = ON       // 電源電圧降下常時監視機能ON(ON)
#pragma config CLKOUTEN = OFF   // CLKOUTﾋﾟﾝをRA6ﾋﾟﾝで使用する(OFF)
#pragma config IESO = OFF       // 外部・内部ｸﾛｯｸの切替えでの起動はなし(OFF)
#pragma config FCMEN = OFF      // 外部ｸﾛｯｸ監視しない(FCMEN_OFF)

// コンフィギュレーション２の設定
#pragma config WRT = OFF        // Flashﾒﾓﾘｰを保護しない(OFF)
#pragma config PLLEN = OFF      // 動作クロックを32MHzでは動作させない(OFF)
#pragma config STVREN = ON      // スタックがオーバフローやアンダーフローしたらリセットをする(ON)
#pragma config BORV = HI        // 電源電圧降下常時監視電圧(2.5V)設定(HI)
#pragma config LVP = OFF        // 低電圧プログラミング機能使用しない(OFF)

#define 	_XTAL_FREQ 8000000 //PICのクロックをHzで設定(8MHz)

/********　変数定義　************/
#define	LINE1			0x80    // １行目設定
#define	LINE2			0xc0    // ２行目設定

#define UARTPRINT       0       //シリアル出力に設定
#define LCDPRINT        1       //ＬＣＤ表示に設定

unsigned	char mojiretu[42];	//文字列用配列
unsigned	int t;              //受信文字数 文字列用			
unsigned	int series_flg;     //文字列完了フラグ

unsigned    int     p_mode;      //出力先切り替え

/******** サブ関数   **************/
void putch(unsigned char byte); //printf用

/*************** サブ関数   **************************/
void LCD_dat(char);                     // １文字表示
void LCD_cmd(char);                     // コマンド出力
void LCD_clr(void);                     // 全消去
void LCD_int(void);                     // 初期化

void contrast_max(void);                //コントラスト調整

/**************** メイン開始  ************************/
void main(void){

    OSCCON = 0b01110000;           // PLL:OFF, 内部クロック8MHzで駆動
    ANSELB = 0b00000000;           // RB7(AN6)-RB1(AN11)は全て0:デジタルI/Oとする
    TRISB  = 0b00010110;           // RB1・RB4 I2C用（入力扱い）
                                   //UART　TX:RB5（出力）/　RX:RB2（入力）　
    
    /* USART初期設定*/
    //　TXCKSEL　0=RB2　1=RB5　　　RXDTSEL　0=RB1　1=RB2
    TXCKSEL = 1;                    // TX = RB5
    RXDTSEL = 1;                    // Rx = RB2
    TXSTA = 0x24;                   //シリアル受信設定
    RCSTA = 0x90;                   //シリアル送信設定
	BAUDCON=0x08;                   //ボードレート設定
	SPBRG = 207;                    //SPBRG=207時9600bps
                                    //SPBRG=103時19200bps
    PEIE = 1;                       //基本モジュール　タイマー０・外部割り込み以外の割り込み許可
	RCIE = 1;                       //受信割り込み許可
	GIE = 1;                        //すべての割り込み許可
	RCIF = 0;                       //シリアル受信割り込みフラグをクリア 

    OPTION_REGbits.nWPUEN = 0;     // 0:PORTB内部プルアップ利用に設定
    WPUBbits.WPUB1 = 1;            // RB1をI2C通信のウィークプルアップ
    WPUBbits.WPUB4 = 1;            // RB4をI2C通信のウィークプルアップ
    SSPCON1 = 0b00101000;          // I2Cマスターモード指定
    SSPSTAT = 0b00000000;          // I2C STATUSの設定
    SSPADD  = 19;                  // I2Cクロック周波数100KHz(=(19+1)*4/8MHz)
    LCD_int();                     // LCD初期化
    contrast_max();                //　コントラスト調整 
    
    // ----------------------------------------------------------
    LCD_cmd(0x0c);                  //　ブリンク非表示
    __delay_ms(2);                  //コマンド送信後は2ms以上のウエイトが必要
    LCD_cmd(0x40);                  //CGRAM　アドレスセット　　 
    __delay_ms(2);                  //コマンド送信後は2ms以上のウエイトが必要
    LCD_clr(); 
    p_mode = LCDPRINT;              //出力先を表示器にする
    LCD_cmd(0x80);                  //1行目に表示 
    __delay_ms(2);
    printf("     Hello !   ");
    LCD_cmd(0xA0);                  //2行目に表示
    __delay_ms(2);
    printf("  2021/01/21    ");

    p_mode = UARTPRINT;             //　出力先をシリアル通信にする
    printf("test  \r\n");
        
    while(1){
  
		if(series_flg == 1)             //文字列受信完了か？
		{
			series_flg = 0;             //フラグをクリア
            p_mode = UARTPRINT;         //　出力先をシリアル通信にする
		
            printf("\r\n rev = %s \r\n",mojiretu);	//文字列を表示
            p_mode = LCDPRINT;              //　出力先を表示器にする
            if (mojiretu[0] == 'L')
			{
                switch(mojiretu[1])	
				{
					case '1':
                        LCD_cmd(0x80);                //1行目に移動 
						printf("                    ");
                        LCD_cmd(0x80);				//1行目の先頭へ
						printf("%s",mojiretu+3);
						break;
						
					case '2':
                        LCD_cmd(0xA0);                //1行目に移動 
						printf("                    ");
                        LCD_cmd(0xA0);				//1行目の先頭へ
						printf("%s",mojiretu+3);
						break;

					default:
                        p_mode = UARTPRINT;         //　出力先をシリアル通信にする
						printf("ERR!\r\n");			//デバッグ用	
						break;
				}   //switch end
			}       // 'L' 時の処理ｅｎｄ		
        }
    }
}

/*************************************************************
 * 受信割り込み関数
 * mojiretu[]へ受信データを移す。
 *改行もしくは復帰があると、（リターンキーの事）
 * その時点でBufferを今まで入力された文字を文字列として扱う。
 *************************************************************/
void interrupt isr(void){

	if (RCREG != 0x0d && RCREG != 0x0a){		//CR　LF 以外か？
		mojiretu[t] = RCREG;		//受信データを文字配列に移動
		t++;

		if(t > 40)				//バッファオーバー時の処理
			t = 0;				//文字配列を初期化
	}
	else{
		mojiretu[t] = 0;		//文字列終了
		series_flg = 1;			//受信完了のフラグを立てる
		t = 0;					//文字配列を初期化
	}
		RCIF = 0;				//割り込みフラグをクリア
}
//-------- １文字表示 --------------------------------------
void LCD_dat(char chr){
    
    CmdI2C(0x78, 0x40, chr);

    __delay_us(60);                // 60μsec
}
//-------- コマンド出力 -----------------------------------------
void LCD_cmd(char cmd){

    CmdI2C(0x78, 0x00, cmd);
    if(cmd & 0xFC)                 // 上位６ビットに１がある命令
        __delay_us(60);            // 60usec
    else
        __delay_ms(3);             // 3msec　ClearおよびHomeコマンド
}
//-------- 全消去 ------------------------------------------------
void LCD_clr(void){
    LCD_cmd(0x01);                 //Clearコマンド出力
}

//-------- 初期化 -----------------------------------------------
void LCD_int(void){
    __delay_ms(400);               // 電源安定するまでの時間
    LCD_cmd(0x01);                 // 8bit 2行 表示命令モード
    __delay_ms(2);
    LCD_cmd(0x02);                 // Display on Cursor=0 Blink=0
    __delay_ms(2);
    LCD_cmd(0x0f);                 // Entry Inc/Dec=1 Shift=0
    __delay_ms(2);
    LCD_cmd(0x01);                 // Clear Display
    __delay_ms(2);
}
//------- コントラスセット　　------------------------------------
void contrast_max(void){
    LCD_cmd(0x2a);      //RE=1
    LCD_cmd(0x79);      //SD=1
    LCD_cmd(0x81);      //コントラストセット
    LCD_cmd(0xFF);      //輝度MAX
    LCD_cmd(0x78);      //SDを0に戻す
    LCD_cmd(0x28);      //2C=高文字　28=ノーマル
    __delay_ms(100);
}

/************************************************************
 * UART/表示器　printf 用 
 ***********************************************************/
void putch(unsigned char byte){
        
    //UART　出力
    if (p_mode == 0 )
    {
		RCSTAbits.SPEN=1;
		TXREG=byte;
		while(!TXIF)continue;
		TXIF=0;
    }   
    //OLED　出力
    else
    {
      LCD_dat(byte);
        
    }
}